//+------------------------------------------------------------------+
//|                                          PriceActionChannels.mq4 |
//|                         Copyright  2009, Forex Tools Cafe       |
//|                                  http://www.forex-tools-cafe.com |
//|  
//|   plots arrow buy sell signals when a bar closes outside the bands 
//|       for the first time since the other side signal occured. 
//|
//|   exports Global Vars , has sound and alerts
//    For voice signals you need to create or purchase a set of wav files that are used to speak the 
//       announcement + Pair name + chart time frame 
//|   
//|   Derived from Moving Average by Metatrader
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, John Burch,  Forex Tools Cafe"
#property link      "http://www.forex-tools-cafe.com"

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 Yellow
#property indicator_color2 Yellow
#property indicator_color3 Yellow
#property indicator_color4 Yellow
#property indicator_width1 2
#property indicator_width2 2

//---- indicator parameters
extern int      MA1_Period=5;
extern int      MA1_Shift=0;
extern int      MA1_Method=2; // smoothed moving average    0 simple, 1 exp ,  2 smoothed, 3, linear weighted

double   V1offset=0.0;

extern int      MA2_Period=5;
extern int      MA2_Shift=0;
extern int      MA2_Method=2;

double   V2offset=0.0;
extern bool     EnableVoice=true;  // voice takes place of simple sound, turn off both for quiet operation
extern bool     EnableSound=false;
extern bool     EnableAlert=false;
extern string   SoundFile_OutsideBar="alert.wav"; // sounds to use when simple sound is desired. 
extern bool     EnableVoiceTimeFrame=true; // can turn off time frame

extern double   MBLslopeLen=4; // depth of slope determination  
extern bool     EnableTDI=true;
extern bool     EnablePrints=false;

int             TrendFlag=-1;

//---- indicator buffers
double Buffer1[];
double Buffer2[];
double buy[];
double sell[];
//----
int ExtCountedBars=0;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   int    draw_begin;
   string short_name;
//---- drawing settings
   SetIndexStyle(0,DRAW_LINE );
   SetIndexShift(0,MA2_Shift);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexShift(1,MA2_Shift);
   
   
   SetIndexArrow(2,'');
   SetIndexArrow(3,'');
  
   SetIndexStyle(2,DRAW_ARROW, EMPTY, 3 , Yellow );
   SetIndexStyle(3,DRAW_ARROW, EMPTY, 3 , Yellow );
   
   
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
   if(MA1_Period<2) MA1_Period=5;
   if(MA2_Period<2) MA2_Period=5;
   draw_begin=MA1_Period-1;
   
//---- indicator short name
   switch(MA1_Method)
   {
      case 1 : short_name="EMA(";  draw_begin=0; break;
      case 2 : short_name="SMMA("; break;
      case 3 : short_name="LWMA("; break;
      default :
         MA1_Method=0;
         short_name="SMA(";
   }
   switch(MA2_Method)
   {
      case 1 : short_name="EMA(";  draw_begin=0; break;
      case 2 : short_name="SMMA("; break;
      case 3 : short_name="LWMA("; break;
      default :
         MA2_Method=0;
         short_name="SMA(";
   }
   IndicatorShortName("PriceActionChannel_Signals"+short_name+MA1_Period+")");
   SetIndexDrawBegin(0,draw_begin);
//---- indicator buffers mapping

   SetIndexBuffer(0,Buffer1);
   SetIndexBuffer(1,Buffer2);
   SetIndexBuffer(2,buy);
   SetIndexBuffer(3,sell); 
   
   TrendFlag=-1;
   
//---- initialization done
   return(0);
}

void deinit()
{

  
}


int start()
{

    if(Bars<=MA1_Period) return(0);
    if(Bars<=MA2_Period) return(0);
   
    ExtCountedBars=IndicatorCounted();
   
    if (ExtCountedBars<0) return(-1);

    if (ExtCountedBars>0) ExtCountedBars--;

    switch(MA1_Method)
    {
       case 0 : sma();  break;
       case 1 : ema();  break;
       case 2 : smma(); break;
       case 3 : lwma();
    }
    switch(MA2_Method)
    {
       case 0 : sma2();  break;
       case 1 : ema2();  break;
       case 2 : smma2(); break;
       case 3 : lwma2();
    }
    
    
    
    
   // BuySell(); // set the arrows for buysell
   
    
   
    double x = (Buffer1[0] + Buffer2[0])/2;
    double y = (Buffer1[5] + Buffer2[5])/2;
    double z = (Buffer1[15] + Buffer2[15])/2;
    double slope5 = x - y ;
    double slope15 = x - z ;
   
    
    
    if ( TrendFlag==-1 && Close[1] > Buffer1[1]  )
    { 
        if ( EnableVoice)  AnnounceSignals(1); 
        else if ( EnableSound )  PlaySound(SoundFile_OutsideBar); 
        if ( EnableAlert )  Alert("PAC: New Trend, SELL");
        
        TrendFlag=1; 
    }
    else if ( TrendFlag==1 && Close[1] < Buffer2[1]  )
    { 
        if ( EnableVoice)  AnnounceSignals(-1); 
        else if ( EnableSound )  PlaySound(SoundFile_OutsideBar); 
        if ( EnableAlert )  Alert("PAC: New Trend, SELL");
        
        TrendFlag=-1; 
    }

    Comment("TrendFlag: " + TrendFlag );
    
    return(0);
}

void AnnounceSignals(int direction )
{
 
    datetime t;

    if ( direction==1 ) 
    {   

        if (   EnablePrints )
            Print("PAC:Going UP: " + Symbol() + " "+ GetPeriodName() ); 

        if (   EnableVoice )
        {
             PlaySound("PriceActionGOINGUP.wav");

             t = TimeLocal(); 
             while( TimeLocal() < t + 3 ) 
             {
                 t = t ;
             }
             PlaySound(SymbolToSound() );

  
            t = TimeLocal(); 
            while( TimeLocal() < t + 2 ) 
            {
                t = t;
            }
     

             if ( EnableVoiceTimeFrame )
            {
                PlaySound( PeriodToSound() );
                t = TimeLocal(); 
                while( TimeLocal() < t + 1 ) 
                    t = t;
            }
          
        }
        else if ( EnableSound )
            PlaySound("alert.wav");
             

        if ( EnableAlert )
            Alert("PAC:Going UP: " + Symbol() + " "+ GetPeriodName() ); 
    } 

    
    if (  direction==-1 ) 
    {    
        
        if (  EnablePrints )
            Print("PAC:Going Down: " + Symbol() + " "+ GetPeriodName() );

        if (  EnableVoice )
        {
             PlaySound("PriceActionGOINGDOWN.wav");

             t = TimeLocal(); 
             while( TimeLocal() < t + 3 ) 
                 t = t ;
     
             PlaySound(SymbolToSound() );

             t = TimeLocal(); 
             while( TimeLocal() < t + 2 ) 
                t = t;

            if ( EnableVoiceTimeFrame )
            {
                PlaySound( PeriodToSound() );
                t = TimeLocal(); 
                while( TimeLocal() < t + 1 ) 
                    t = t;
            }
            
        }
        else if ( EnableSound )
            PlaySound("alert.wav");
            
            
        if (  EnableAlert )
             Alert("PAC:Going Down: " + Symbol() + " "+ GetPeriodName() ); 
                
    }
}




void BuySell()
{
    double sum=0;
    int    i,pos=Bars-ExtCountedBars-2;


    while(pos>=0)
    {

    
        double pl = iCustom(Symbol(),Period(),
        "TradersDynamicIndex_TradeAlerts", 
        13, 0,  34, 2, 0, 7, 0, false, false, "alert.wav",
        4, // index number
        pos ); // shift
    
        double tsl = iCustom(Symbol(),Period(),
        "TradersDynamicIndex_TradeAlerts", 
        13, 0,  34, 2, 0, 7, 0, false, false, "alert.wav",
        5, // index number
        pos ); // shift
    
        double mbl = iCustom(Symbol(),Period(),
        "TradersDynamicIndex_TradeAlerts", 
        13, 0,  34, 2, 0, 7, 0, false, false, "alert.wav",
        2, // index number
        pos ); // shift
    
        double mbl_slope  = mbl - iCustom(Symbol(),Period(),
        "TradersDynamicIndex_TradeAlerts", 
        13, 0,  34, 2, 0, 7, 0, false, false, "alert.wav",
        2, // index number
        pos+MBLslopeLen ); // shift
    
        int buysell = 0;
    
        if ( pl > 45 && pl > tsl && pl > mbl && mbl_slope >= 0.6 && mbl < 57 )
            buysell = 1; // buy is ok 
        else if ( pl < 55 && pl < tsl && pl < mbl && mbl_slope <= -0.6 && mbl > 43 )
            buysell = -1;     
    
    
        if ( TrendFlag==-1 && Close[pos+1] > Buffer1[pos+1]  )//  buysell==1 &&
        { 
            if ( EnableTDI  )
            {  
                if (  buysell==1 )
                    buy[pos] = Low[pos] - 0.0008 ;
            }
            else   
                buy[pos] = Low[pos] - 0.0008 ;
                
            TrendFlag=1; 
        }
        else if ( TrendFlag==1 && Close[pos +1] < Buffer2[pos+1] )// buysell==-1 &&
        { 
            if ( EnableTDI    ) 
            {
                if ( buysell==-1 )
                    sell[pos] = High[pos] + 0.0008;
            }
            else
                 sell[pos] = High[pos] + 0.0008;
        
            TrendFlag=-1; 
        }

        pos--;
    }
    
  /*   Comment(              "green pl: " + pl 
                        + "\nred tsl: " + tsl 
                        + "\nyellow MBL: " + mbl 
                        + "\nSlope: " + mbl_slope
                        + "\nBuySell: " + buysell  );
    */

}



string GetPeriodName()
{

    int peri = Period();
    string perstr; 
    switch(peri) {
        case PERIOD_M1: { perstr="M1"  ; break;}
        case PERIOD_M5: { perstr="M5"  ; break;}
        case PERIOD_M15: {perstr="M15" ; break;}
        case PERIOD_M30: {perstr="M30" ; break;}
        case PERIOD_H1:  {perstr="H1"  ; break;}
        case PERIOD_H4:  {perstr="H4"  ; break;}
        case PERIOD_D1:  {perstr="D1"  ; break;}
        case PERIOD_W1:  {perstr="W1"  ; break;}
        case PERIOD_MN1: {perstr="MN1";}
     } 
     return (perstr);
}

string SymbolToSound()
{

    string  pairs[]={"EURUSD","GBPUSD","AUDUSD","USDCHF","USDJPY","EURAUD","GBPCHF","GBPJPY","EURJPY","USDCAD"};

    string sound;
    int  k; 
    string  sym = Symbol();
    for ( int i=0; i<ArraySize(pairs); i++ )
    {
        if ( pairs[i] ==  sym )
            k= i; 
    }
    switch(k) {
       case 0:  sound="EUR_USD.wav"  ; break;
       case 1:  sound="GBP_USD.wav"  ; break;
       case 2: sound="AUD_USD.wav" ; break;
       case 3: sound="USD_CHF.wav" ; break;
       case 4:  sound="USD_JPY.wav"  ; break;
       case 5:  sound="EUR_AUD.wav"  ; break;
       case 6:  sound="GBP_CHF.wav"  ; break;
       case 7:  sound="GBP_JPY.wav"  ; break;
       case 8: sound="EUR_JPY.wav";break;
       case 9: sound="USD_CAD.wav";break;
       default:
            sound = "alert.wav"; 
    }
    return(sound);
}

string PeriodToSound()
{
   string sound;
   switch(Period()) {
      case PERIOD_M1:  sound="M1.wav"  ; break;
      case PERIOD_M5:  sound="M5.wav"  ; break;
      case PERIOD_M15: sound="M15.wav" ; break;
      case PERIOD_M30: sound="M30.wav" ; break;
      case PERIOD_H1:  sound="H1.wav"  ; break;
      case PERIOD_H4:  sound="H4.wav"  ; break;
      case PERIOD_D1:  sound="DAILY.wav"  ; break;
      case PERIOD_W1:  sound="WEEKLY.wav"  ; break;
      case PERIOD_MN1: sound="MONTHLY.wav";
   }
   return(sound);
}



//+------------------------------------------------------------------+
//| Simple Moving Average                                            |
//+------------------------------------------------------------------+

void sma()
{
   double sum=0;
   int    i,pos=Bars-ExtCountedBars-1;

   if(pos<MA1_Period) pos=MA1_Period;
   for(i=1;i<MA1_Period;i++,pos--)
      sum+=High[pos];

   while(pos>=0)
     {
      sum+=High[pos];
      Buffer1[pos]=(sum/MA1_Period) + V1offset;
	   sum-=High[pos+MA1_Period-1];
 	   pos--;
     }

   if(ExtCountedBars<1)
      for(i=1;i<MA1_Period;i++) Buffer1[Bars-i]=0;
}
  
void sma2()
{
   double sum=0;
   int    i,pos=Bars-ExtCountedBars-1;
//---- initial accumulation
   if(pos<MA2_Period) pos=MA2_Period;
   for(i=1;i<MA2_Period;i++,pos--)
      sum+=Low[pos];
//---- main calculation loop
   while(pos>=0)
     {
      sum+=Low[pos];
      Buffer2[pos]=( sum/MA2_Period) - V2offset;
	   sum-=Low[pos+MA2_Period-1];
 	   pos--;
     }
//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<MA2_Period;i++) Buffer2[Bars-i]=0;
}  
  
  
//+------------------------------------------------------------------+
//| Exponential Moving Average                                       |
//+------------------------------------------------------------------+
void ema()
{
   double pr=2.0/(MA1_Period+1);
   int    pos=Bars-2;
   if(ExtCountedBars>2) pos=Bars-ExtCountedBars-1;
//---- main calculation loop
   while(pos>=0)
   {
      if(pos==Bars-2) Buffer1[pos+1]= V1offset + High[pos+1];
      Buffer1[pos]=  (High[pos]*pr+Buffer1[pos+1]*(1-pr)) + V1offset;
 	  pos--;
   }
}

void ema2()
{
   double pr=2.0/(MA2_Period+1);
   int    pos=Bars-2;
   if(ExtCountedBars>2) pos=Bars-ExtCountedBars-1;
//---- main calculation loop
   while(pos>=0)
   {
      if(pos==Bars-2) Buffer2[pos+1]=  Low[pos+1] - V2offset ;
      Buffer2[pos]= (Low[pos]*pr+Buffer2[pos+1]*(1-pr) )- V2offset  ;
 	  pos--;
   }
}


//+------------------------------------------------------------------+
//| Smoothed Moving Average                                          |
//+------------------------------------------------------------------+
void smma()
{
   double sum=0;
   int    i,k,pos=Bars-ExtCountedBars+1;
//---- main calculation loop
   pos=Bars-MA1_Period;
   if(pos>Bars-ExtCountedBars) pos=Bars-ExtCountedBars;
   while(pos>=0)
   {
      if(pos==Bars-MA1_Period)
      {
         //---- initial accumulation
         for(i=0,k=pos;i<MA1_Period;i++,k++)
         {
            sum+=High[k];
            //---- zero initial bars
            Buffer1[k]=0;
         }
      }
      else sum=Buffer1[pos+1]*(MA1_Period-1)+High[pos];
      Buffer1[pos]=sum/MA1_Period;
 	  pos--;
   }
}
  
void smma2()
{
   double sum=0;
   int    i,k,pos=Bars-ExtCountedBars+1;
//---- main calculation loop
   pos=Bars-MA2_Period;
   if(pos>Bars-ExtCountedBars) pos=Bars-ExtCountedBars;
   while(pos>=0)
   {
      if(pos==Bars-MA2_Period)
      {
         //---- initial accumulation
         for(i=0,k=pos;i<MA2_Period;i++,k++)
         {
            sum+=Low[k];
            //---- zero initial bars
            Buffer2[k]=0;
         }
      }
      else sum=Buffer2[pos+1]*(MA2_Period-1)+Low[pos];
      Buffer2[pos]=sum/MA2_Period;
 	  pos--;
   }
}


//+------------------------------------------------------------------+
//| Linear Weighted Moving Average                                   |
//+------------------------------------------------------------------+
void lwma()
{
   double sum=0.0,lsum=0.0;
   double price;
   int    i,weight=0,pos=Bars-ExtCountedBars-1;
//---- initial accumulation
   if(pos<MA1_Period) pos=MA1_Period;
   for(i=1;i<=MA1_Period;i++,pos--)
   {
      price=High[pos];
      sum+=price*i;
      lsum+=price;
      weight+=i;
   }
//---- main calculation loop
   pos++;
   i=pos+MA1_Period;
   while(pos>=0)
   {
      Buffer1[pos]=sum/weight;
      if(pos==0) break;
      pos--;
      i--;
      price=High[pos];
      sum=sum-lsum+price*MA1_Period;
      lsum-=High[i];
      lsum+=price;
   }
//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<MA1_Period;i++) Buffer1[Bars-i]=0;
}

void lwma2()
{
   double sum=0.0,lsum=0.0;
   double price;
   int    i,weight=0,pos=Bars-ExtCountedBars-1;
//---- initial accumulation
   if(pos<MA2_Period) pos=MA2_Period;
   for(i=1;i<=MA2_Period;i++,pos--)
   {
      price=Low[pos];
      sum+=price*i;
      lsum+=price;
      weight+=i;
   }
//---- main calculation loop
   pos++;
   i=pos+MA2_Period;
   while(pos>=0)
   {
      Buffer2[pos]=sum/weight;
      if(pos==0) break;
      pos--;
      i--;
      price=Low[pos];
      sum=sum-lsum+price*MA2_Period;
      lsum-=Low[i];
      lsum+=price;
   }
//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<MA2_Period;i++) Buffer2[Bars-i]=0;
}

//+------------------------------------------------------------------+

